<?php
	
	namespace Fortvision\FortvisionEventTracker;
	
	if (!defined('ABSPATH')) exit();
	
	class FortvisionEventTracker{
		protected static $instance = null;
		protected static $uidCookieName = 'cms_fortvision_uid';
		protected $options = array();
		protected $userId = '';
		public $curActions = array();
		protected $routUrl = 'https://fb.fortvision.com/fb';
		protected $routUrl_debug = 'https://fbdev.fortvision.com:4443';
		protected $routes = array(
			'addToCart' => '/cart-management/product/add',
			'removeFromCart' => '/cart-management/product/remove',
			'updateCheckoutStatus' => '/cart-management/cart/checkout',
			'updateCart' => '/cart-management/cart/status',
			'login' => '/users/login',
			'addUserToSubscription' => '/users/subscription',
			'register' => '/users/register',
		);
		protected $isWoocommerce = false;
		public $sentActions = array();
		
		public static function instance() {
			if (is_null(self::$instance)) {
				self::$instance = new self();
			}
			
			return self::$instance;
		}
		
		public function __construct() {
			$this->init();
			$this->register_hooks();
		}
		
		public function register_hooks() {
			add_action('init', array($this,'action_init'));
			
			add_action('rest_api_init', array(
				$this,
				'initAPIRoutes'
			));
			
			add_action('wp_login', array(
				$this,
				'action_fortvision__wp_login'
			), 11, 2);
			add_action('woocommerce_add_to_cart', array(
				&$this,
				'action_fortvision__woocommerce_add_to_cart'
			), 10, 6);
			add_action('woocommerce_remove_cart_item', array(
				&$this,
				'action_fortvision__woocommerce_remove_cart_item'
			), 10, 2);
			add_action('woocommerce_cart_item_set_quantity', array(
				&$this,
				'action_fortvision__woocommerce_cart_item_set_quantity'
			), 10, 3);
			add_action('woocommerce_order_status_changed', array(
				&$this,
				'action_fortvision__woocommerce_order_status_changed'
			), 10, 3);
			add_action('user_register', array(
				$this,
				'action_fortvision__user_register'
			), 11, 2);
			
			add_filter('rocket_cache_wc_empty_cart', function () {
				
				return false;
			});
/*			
			if (!isset($this->options['subscriptions']['when_to_report_a_purchase'])) {
				add_action('woocommerce_thankyou',  array($this,'fortvisionReportThankYou'));
				add_action('woocommerce_new_order', array($this,'fortvisionReportNewOrder'));
			} else {
				if($this->options['subscriptions']['when_to_report_a_purchase'] != 'when_order_has_been_placed'){
					add_action('woocommerce_thankyou',  array($this,'fortvisionReportThankYou'));
				} else {
					add_action('woocommerce_new_order', array($this,'fortvisionReportNewOrder'));
				}
			}
*/
			//fortvisionAddCheckBoxMarketing from register form
			add_action('woocommerce_created_customer', array($this,'fortvisionNewCustomer'), PHP_INT_MAX, 3);
			//fortvisionAddCheckBoxMarketing from checkout page
			add_action('woocommerce_new_order', array($this,'fortvisionNewOrder'));
		}
		
		public function action_init() {
			$this->isWoocommerce = is_plugin_active('woocommerce/woocommerce.php');
		}
		
		public function initAPIRoutes() {
			register_rest_route('fv/v1', 'logs_enable', array(
					'methods' => array('GET'),
					'callback' => array(
						$this,
						'API_logsEnabled'
					),
					'permission_callback' => '__return_true'
				));
			register_rest_route('fv/v1', 'debug_enable', array(
					'methods' => array('GET'),
					'callback' => array(
						$this,
						'API_debugEnabled'
					),
					'permission_callback' => '__return_true'
				));
			register_rest_route('fv/v1', 'logs_get', array(
					'methods' => array('GET'),
					'callback' => array(
						$this,
						'API_logsGet'
					),
					'permission_callback' => '__return_true'
				));
			register_rest_route('fv/v1', 'logs_clear', array(
					'methods' => array('GET'),
					'callback' => array(
						$this,
						'API_logsClear'
					),
					'permission_callback' => '__return_true'
				));
			register_rest_route('fv/v1', 'set_uid', array(
					'methods' => array(
						'POST',
						'GET'
					),
					'callback' => array(
						$this,
						'API_setUid'
					),
					'permission_callback' => '__return_true'
				));
		}
		
		public function API_logsEnabled($request_data) {
			$parameters = $request_data->get_params();
			if (isset($parameters['enable'])) {
				update_option('fortvision_debug_mode_enabled', sanitize_text_field((int)$parameters['enable']));
				$this->options['debug_mode_enabled'] = get_option('fortvision_debug_mode_enabled', '');
				
				return rest_ensure_response(array('success' => true));
			}
			
			return rest_ensure_response(array('success' => false));
		}
		
		public function API_debugEnabled($request_data) {
			$parameters = $request_data->get_params();
			if (isset($parameters['enable'])) {
				update_option('fortvision_debug_mode_enabled_requests', sanitize_text_field((int)$parameters['enable']));
				$this->options['debug_mode_enabled_requests'] = get_option('fortvision_debug_mode_enabled_requests', '');
				
				return rest_ensure_response(array('success' => true));
			}
			
			return rest_ensure_response(array('success' => false));
		}
		
		public function API_logsGet($request_data) {
			$uploadDir = wp_upload_dir(null, false);
			$logBaseDir = $uploadDir['basedir'] . '/fortvision-logs/';
			$logFile = $logBaseDir . 'requests.log';
			$file = @file_get_contents($logFile);
			echo 'Version ID: ' . FTVSN_PLUGIN_VER . "\n";
			echo 'Publish ID: ' . $this->options['fortvision_script_val'] . "\n";
			echo 'Logs: ' . (($this->options['debug_mode_enabled']) ? 'on' : 'off') . "\n";
			echo 'Debug requests: ' . (($this->options['debug_mode_enabled_requests']) ? 'on' : 'off') . "\n";
			echo 'Log size:' . @filesize($logFile) . "\n\n";
			echo $file;
			exit();
		}
		
		public function API_logsClear($request_data) {
			$uploadDir = wp_upload_dir(null, false);
			$logBaseDir = $uploadDir['basedir'] . '/fortvision-logs/';
			$logFile = $logBaseDir . 'requests.log';
			if (file_exists($logFile)) {
				@unlink($logFile);
			}
		}
		
		public function API_setUid($request_data) {
			$parameters = $request_data->get_params();
			$response = array('success' => false);
			if (isset($parameters['uid']) && $parameters['uid']) {
				$this->userId = $parameters['uid'];
				setcookie(self::$uidCookieName, sanitize_text_field($parameters['uid']), strtotime('+30 days'), '/');
				
				$response = array('success' => true);
			}
			$logData = array(
				'Time' => date('Y-m-d H:i:s'),
				'Action' => 'setUid',
				'Data' => $parameters,
				'Response' => $response,
			);
			$this->_log($logData);
			
			return rest_ensure_response($response);
		}
		
		private function init() {
			$this->options['fortvision_script_val'] = trim(get_option('fortvision_script_val', ''));
			$this->options['debug_mode_enabled'] = get_option('fortvision_debug_mode_enabled', '');
			$this->options['debug_mode_enabled_requests'] = get_option('fortvision_debug_mode_enabled_requests', '');
			$this->options['subscriptions'] = json_decode(get_option('_fortvision_form', '{}'), true);
			
			if (isset($_COOKIE[self::$uidCookieName]) && $_COOKIE[self::$uidCookieName]) {
				$this->userId = $_COOKIE[self::$uidCookieName];
			}
			
			$this->logRotate();
		}
				
		public function action_fortvision__wp_login($user_login, $login) {
			$userInfo = array(
				'isLoggedIn' => 1,
				'firstName' => get_user_meta($login->id, 'first_name', true),
				'lastName' => get_user_meta($login->id, 'last_name', true),
				'phone' => (string)get_user_meta($login->id, 'billing_phone', true),
				'email' => $login->user_email,
			);
			$this->setAction('login', array(
				'user_login' => $user_login,
				'login' => $login,
				'userInfo' => $userInfo,
			));
		}
		
		public function action_fortvision__user_register($user_id, $userdata) {
			$current_user = wp_get_current_user();
			$user = get_user_by('id', $user_id);
			$userInfo = array(
				'isLoggedIn' => ($current_user->ID) ? 1 : 0,
				'firstName' => get_user_meta($user_id, 'first_name', true),
				'lastName' => get_user_meta($user_id, 'last_name', true),
				'phone' => (string)get_user_meta($user_id, 'billing_phone', true),
				'email' => $user->user_email,
			);
			$this->setAction('register', array(
				'user_id' => $user_id,
				'userdata' => $userdata,
				'userInfo' => $userInfo,
			));
		}
		
		public function action_fortvision__woocommerce_add_to_cart($cart_item_key, $product_id, $quantity, $variation_id, $variation, $cart_item_data) {
			$this->setAction('addToCart', array(
				'cart_item_key' => $cart_item_key,
				'product_id' => $product_id,
				'quantity' => $quantity,
				'variation_id' => $variation_id,
				'variation' => $variation,
				'cart_item_data' => $cart_item_data,
				'status' => 'add',
			));
		}
		
		public function action_fortvision__woocommerce_remove_cart_item($cart_item_key, $object) {
			$this->setAction('removeFromCart', array(
				'cart_item_key' => $cart_item_key,
				'object' => $object,
				'status' => 'remove',
			));
		}
		
		public function action_fortvision__woocommerce_cart_item_set_quantity($cart_item_key, $quantity, $object) {
			$this->setAction('updateCart', array(
				'cart_item_key' => $cart_item_key,
				'quantity' => $quantity,
				'object' => $object,
				'status' => 'update',
			));
		}
		
		public function action_fortvision__woocommerce_order_status_changed($order_id, $previous_status, $next_status) {
			$this->setAction('updateCheckoutStatus', array(
				'order_id' => $order_id,
				'previous_status' => $previous_status,
				'next_status' => $next_status
			));
		}
		
		function fortvisionNewOrder($order_id) {
			if (isset($this->options['subscriptions']['which_pages']) && in_array($this->options['subscriptions']['which_pages'], array('on_checkout_page', 'both'))) {
				
				$order = wc_get_order($order_id);
				$customer = array();
				$customer['first_name'] = $order->get_billing_first_name();
				$customer['last_name'] = $order->get_billing_last_name();
				$customer['phone'] = (string)$order->get_billing_phone();
				$customer['email'] = $order->get_billing_email();
				$customer['city'] = $order->get_billing_city();
				$customer['country'] = $order->get_billing_country();
				$guest = false;
				if ($order->get_user()) {
					$user = $order->get_user();
					$customer['user_id'] = $user->ID;
				} else {
					$guest = true;
					$order_meta = get_post_meta($order_id);
				}
				
				$meta = ($guest != true) ? get_user_meta($customer['user_id']) : array();
				
				$_subscribe = '_fortvisionAcceptMarketing';
				$accept = false;
				if (isset($_POST[$_subscribe])) {
					$accept = $_POST[$_subscribe];
				} else {
					if ($guest == true && isset($order_meta[$_subscribe][0])) {
						$accept = $order_meta[$_subscribe][0];
					} else {
						if ($guest == false && isset($meta[$_subscribe][0])) {
							$accept = $meta[$_subscribe][0];
						} else {
							$accept = false;
						}
					}
				}
				
				if (strval($accept) == '1' || strval($accept) == 'yes') {
					$create = $this->fortvisionSubscribe($customer);
				}
			}
		}
		
		function fortvisionNewCustomer($customer_id, $customer_data) {
			if (isset($this->options['subscriptions']['which_pages']) && in_array($this->options['subscriptions']['which_pages'], array('on_signup_page','both'))) {
				$user = get_userdata($customer_id);
				$meta = get_user_meta($customer_id);
				
				$customer = array(
					'email' => $user->data->user_email,
					'user_id' => $user->data->ID
				);
				
				if (isset($meta['first_name'][0])) $customer['first_name'] = $meta['first_name'][0];
				if (isset($meta['last_name'][0])) $customer['last_name'] = $meta['last_name'][0];
				
				if (isset($meta['billing_phone'][0])) $customer['phone'] = $meta['billing_phone'][0];
				if (isset($meta['shipping_city'][0])) $customer['city'] = $meta['shipping_city'][0];
				
				if (isset($meta['billing_country'][0])) $customer['country'] = $meta['billing_country'][0];
				
				$_subscribe = '_fortvisionAcceptMarketing';
				if (isset($_POST[$_subscribe])) {
					$accept = $_POST[$_subscribe];
				} else {
					$accept = (isset($meta[$_subscribe][0])) ? $meta[$_subscribe][0] : false;
				}
				
				$subscribe = (strval($accept) == '1' || strval($accept) == 'yes') ? true : false;
				
				if (strval($accept) == '1' || strval($accept) == 'yes') {
					$create = $this->fortvisionSubscribe($customer);
				}
			}
		}
		
		function fortvisionSubscribe($customer) {
			if (isset($customer['user_id']) && is_numeric($customer['user_id'])) {
				update_user_meta($customer['user_id'], '_fortvisionAcceptMarketing', 1);
			}
			
			$addSubscribtionParams = array(
				//'city' => $customer['city'],
				//'country' => $customer['country'],
				'userInfo' => array(
					'firstName' => $customer['first_name'],
					'lastName' => $customer['last_name'],
					'email' => $customer['email'],
					'phone' => isset($customer['phone']) ? (string)$customer['phone'] : ''
				),
				//: UserInfoParams;
				'subscriptionType' => 1
			);
			$this->setAction('addUserToSubscription', array('customer' => $addSubscribtionParams));
		}
/*		
		function fortvisionReportThankYou($order_id) {
			$this->setAction('updateCheckoutStatus', array('order_id' => $order_id));
		}
		
		function fortvisionReportNewOrder($order_id) {
			$this->setAction('updateCheckoutStatus', array('order_id' => $order_id));
		}
*/
		public function setAction($curAction, $itemParams) {
			if (!$this->isWoocommerce)
				return false;
			
			if (in_array($curAction, self::instance()->sentActions))
				return false;
			
			self::instance()->sentActions[] = $curAction;
			
			$this->curActions[$curAction] = $itemParams;
			
			$this->prepareData();
		}
		
		public function prepareData() {
			foreach ($this->curActions as $action => $itemParams) {
				$finalData = $return = array();
				switch ($action) {
					case 'addUserToSubscription':
						$current_user = wp_get_current_user();
						$return['subscriptionType'] = $itemParams['customer']['subscriptionType'];
						//$return['city'] = $itemParams['customer']['city'];
						//$return['country'] = $itemParams['customer']['country'];
						$return['userInfo'] = $itemParams['customer']['userInfo'];
						$return['userInfo']['publisherId'] = $this->options['fortvision_script_val'];
						$return['userInfo']['userId'] = $this->userId;
						$return['userInfo']['isLoggedIn'] = ($current_user->ID) ? 1 : 0;
						break;
					case 'login':
						$return['userInfo'] = isset($itemParams['userInfo']) ? $itemParams['userInfo'] : self::instance()->getUserInfo();
						$return['userInfo']['publisherId'] = $this->options['fortvision_script_val'];
						$return['userInfo']['userId'] = $this->userId;
						break;
					case 'register':
						$return['userInfo'] = isset($itemParams['userInfo']) ? $itemParams['userInfo'] : self::instance()->getUserInfo();
						$return['userInfo']['publisherId'] = $this->options['fortvision_script_val'];
						$return['userInfo']['userId'] = $this->userId;
						break;
					case 'addToCart':
						$finalData = $this->getParamsOnAdd($itemParams);
						
						$return['cart'] = $this->getCartParams('add');
						$return['userInfo'] = isset($finalData['userInfo']) ? $finalData['userInfo'] : self::instance()->getUserInfo();;
						$return['product'] = $finalData['product'];
						$return['volume'] = $finalData['volume'];
						break;
					case 'removeFromCart':
						$finalData = $this->getParamsOnRemove($itemParams);
						
						$return['cart'] = $this->getCartParams('remove');
						$return['userInfo'] = isset($finalData['userInfo']) ? $finalData['userInfo'] : self::instance()->getUserInfo();;
						$return['product'] = $finalData['product'];
						$return['volume'] = $finalData['volume'];
						break;
					case 'updateCart':
						$finalData = $this->getParamsOnUpdated($itemParams);
						
						$return['cart'] = $this->getCartParams('updated');
						$return['userInfo'] = isset($finalData['userInfo']) ? $finalData['userInfo'] : self::instance()->getUserInfo();;
						//$return['product'] = $finalData['product'];
						//$return['volume'] = $finalData['volume'];
						break;
					
					case 'updateCheckoutStatus':
						$return = $this->getOrderInfo($itemParams);
						break;
				}
				
				if ($this->options['debug_mode_enabled_requests']) {
					$return['validation'] = 'true';
				}
				
				$this->sendRequest($action, $return);
			}
			
			$this->curActions = array();
		}
		
		public function getUserInfo() {
			$current_user = wp_get_current_user();
			$userInfo['isLoggedIn'] = ($current_user->ID) ? 1 : 0;
			$userInfo['firstName'] = $current_user->user_firstname;
			$userInfo['lastName'] = $current_user->user_lastname;
			$userInfo['email'] = $current_user->user_email;
			$userInfo['phone'] = get_user_meta($current_user->ID, 'billing_phone', true);
			$userInfo['publisherId'] = $this->options['fortvision_script_val'];
			$userInfo['userId'] = $this->userId;
			
			return $userInfo;
		}
		
		public function getParamsOnAdd($itemParams) {
			$product_final = $product_action = array();
			$product_id = $itemParams['product_id'];
			$variation_id = $itemParams['variation_id'];
			$quantity = empty($itemParams['quantity']) ? 1 : wc_stock_amount($itemParams['quantity']);
			
			$product_action = $this->getProductInfo($product_id, $variation_id);
			
			$product_final = array(
				'product' => $product_action,
				'volume' => $quantity
			);
			
			return $product_final;
		}
		
		public function getParamsOnRemove($itemParams) {
			$product_action = $product_final = array();
			$quantity = (isset($itemParams['quantity'])) ? $itemParams['quantity'] : 0;
			$product_id = (isset($itemParams['product_id'])) ? $itemParams['product_id'] : 0;
			$variation_id = (isset($itemParams['variation_id'])) ? $itemParams['variation_id'] : 0;
			$cart_item_key_ajax = isset($itemParams['cart_item_key']) ? $itemParams['cart_item_key'] : '';
			
			if (isset($itemParams['object']) && is_object($itemParams['object']) && isset($itemParams['object']->removed_cart_contents[$cart_item_key_ajax]) && $itemParams['object']->removed_cart_contents[$cart_item_key_ajax]) {
				$product_action_deleted = $itemParams['object']->removed_cart_contents[$cart_item_key_ajax];
				if (!$quantity) {
					$quantity = $product_action_deleted['quantity'];
				}
				$product_action = $this->getProductInfo($product_action_deleted['product_id'], $product_action_deleted['variation_id']);
			}
			
			$product_final = array(
				'product' => $product_action,
				'volume' => $quantity,
			);
			
			return $product_final;
		}
		
		public function getParamsOnUpdated($itemParams) {
			$product_action = $product_final = array();
			foreach (WC()->cart->get_cart() as $cart_item_key => $cart_item) {
				$fort_product = $this->getCartItemProductInfo($cart_item);
				if (isset($itemParams['cart_item_key']) && $itemParams['cart_item_key'] = $cart_item_key) {
					$product_action = $fort_product;
				}
			}
			
			$product_final = array(
				'product' => $product_action,
				'volume' => isset($product_action['count']) ? $product_action['count'] : 0,
			);
			
			return $product_final;
		}
		
		public function getCartParams($cmsStatus) {
			$cart_item_all = $cart_products_set = array();
			$cw_discount = $volume = 0;
			
			foreach (WC()->cart->get_cart() as $cart_item_key => $cart_item) {
				$fort_product = $this->getCartItemProductInfo($cart_item);
				$cw_discount += $fort_product['discountValue'] * $fort_product['count'];
				$volume += $fort_product['count'];
				$cart_item_all[] = $fort_product;
			}
			
			$cart_products_set['products'] = $cart_item_all;
			$cart_products_set['discountValue'] = $cw_discount;
			$cart_products_set['discountedValue'] = WC()->cart->total;
			$cart_products_set['volume'] = $volume;
			$cart_products_set['cmsStatus'] = $cmsStatus;
			if (WC()->cart->get_applied_coupons()) {
				$cart_title = WC()->cart->get_applied_coupons();
				$cart_products_set['couponId'] = $cart_title[0];
			}
			
			return $cart_products_set;
		}
		
		public function getProductInfo($productId, $variationId = false) {
			$fort_product = $productCategory = array();
			$productName = get_the_title($productId);
			$excerpt = get_the_excerpt($productId);
			$productDiscription = ((isset($excerpt) && !empty($excerpt)) ? $excerpt : '');
			if ($variationId) {
				$variable_product = wc_get_product($variationId);
				$productPrice = (float)$variable_product->get_price();
				$productSku = $variable_product->get_sku();
				$productBrand = $variable_product->get_meta('pa_brand', true);
				$stockQty = $variable_product->get_stock_quantity();
			} else {
				$simple_product = wc_get_product($productId);
				$productPrice = (float)$simple_product->get_price();
				$productSku = $simple_product->get_sku();
				$productBrand = $simple_product->get_meta('pa_brand', true);
				$stockQty = $simple_product->get_stock_quantity();
			}
			
			$terms = wp_get_post_terms($productId, 'product_cat');
			if (count($terms) > 0) {
				foreach ($terms as $term) {
					$term_name = $term->name;
					$productCategory[] = $term_name;
				}
			}
			$productCategory = implode(',', $productCategory);
			$productCurrency = get_woocommerce_currency();
			$fort_product_image = wp_get_attachment_image_src(get_post_thumbnail_id($productId), 'single-post-thumbnail');
			$productImageUrl = isset($fort_product_image[0]) ? $fort_product_image[0] : '';
			$productUrl = get_permalink($productId);
			
			$fort_product['customerProductId'] = $productId;
			$fort_product['name'] = $productName;
			$fort_product['discountedValue'] = $productPrice;
			$fort_product['category'] = $productCategory;
			$fort_product['currency'] = $productCurrency;
			$fort_product['description'] = $productDiscription;
			$fort_product['sku'] = $productSku;
			$fort_product['imageUrl'] = $productImageUrl;
			$fort_product['productUrl'] = $productUrl;
			$fort_product['brand'] = $productBrand;
			$fort_product['categories'] = $this->getTaxonomy($productId, 'product_cat');
			$fort_product['attributes'] = $this->getAttributes($productId);
			$fort_product['tags'] = $this->getTaxonomy($productId, 'product_tag');
			$fort_product['stock'] = $stockQty;
			
			return $fort_product;
		}
		
		public function getCartItemProductInfo($cartItem) {
			$discount = 0;
			$fort_product = $this->getProductInfo($cartItem['product_id'], $cartItem['variation_id']);
			$productCount = $cartItem['quantity'];
			$_product = $cartItem['data'];
			if ($_product->is_on_sale()) {
				$regular_price = (float)$_product->get_regular_price();
				$sale_price = (float)$_product->get_sale_price();
				$discount = ($regular_price - $sale_price);
			}
			
			$fort_product['discountValue'] = $discount;
			$fort_product['count'] = $productCount;
			
			return $fort_product;
		}
		
		public function getOrderInfo($itemParams) {
			$return = $order_cartparamas = $main_product_cart_params = array();
			$count = 0;
			$order_id = $itemParams['order_id'];
			$order = wc_get_order($order_id);
			foreach ($order->get_items() as $item_id => $item) {
				$ordered_products = array();
				$ordered_products['customerProductId'] = $item->get_product_id();
				$product_variation_id = $item->get_variation_id();
				$variable_product = wc_get_product($product_variation_id);
				$ordered_products['name'] = $item->get_name();
				$product = $item->get_product();
				$ordered_products['count'] = $item->get_quantity();
				$count = $count + $item->get_quantity();
				if ($item->get_variation_id() != 0) {
					$ordered_products['discountedValue'] = $variable_product->get_price();
					$stockQty = $variable_product->get_stock_quantity();
				} else {
					$main_product = wc_get_product($ordered_products['customerProductId']);
					$ordered_products['discountedValue'] = $main_product->get_price();
					$stockQty = $main_product->get_stock_quantity();
				}
				$ordered_products['brand'] = $item->get_meta('pa_brand', true);
				$ordered_products['currency'] = get_woocommerce_currency();
				$terms = wp_get_post_terms($item->get_product_id(), 'product_cat');
				if (count($terms) > 0) {
					foreach ($terms as $term) {
						$term_name = $term->name;
						$output[] = $term_name;
					}
					$ordered_products['category'] = implode(', ', $output);
				}
				
				$fort_product_image = wp_get_attachment_image_src(get_post_thumbnail_id($item->get_product_id()), 'single-post-thumbnail');
				$productImageUrl = isset($fort_product_image[0]) ? $fort_product_image[0] : '';
				$productUrl = get_permalink($item->get_product_id());
				
				$ordered_products['productUrl'] = $productUrl;
				$ordered_products['sku'] = $product->get_sku();
				$ordered_products['imageUrl'] = $productImageUrl;
				$ordered_products['currency'] = get_woocommerce_currency();
				
				$ordered_products['categories'] = $this->getTaxonomy($item->get_product_id(), 'product_cat');
				$ordered_products['attributes'] = $this->getAttributes($item->get_product_id());
				$ordered_products['tags'] = $this->getTaxonomy($item->get_product_id(), 'product_tag');
				$ordered_products['stock'] = $stockQty;
				
				$main_product_cart_params[] = $ordered_products;
			}
			$order_cartparamas['products'] = $main_product_cart_params;
			$coupon_array = $order->get_coupon_codes();
			if (isset($coupon_array) && !empty($coupon_array)) {
				$order_cartparamas['couponId'] = $coupon_array[0];
			}
			$order_cartparamas['discountValue'] = $order->get_total_discount();
			$order_cartparamas['discountedValue'] = $order->get_total();
			$order_cartparamas['volume'] = $count;
			$order_cartparamas['cmsStatus'] = $order->get_status();
			$order_cartparamas['checkoutId'] = (string)$order_id;
			
			$current_user = wp_get_current_user();
			$userInfo = array(
				'firstName' => $order->get_billing_first_name(),
				'lastName' => $order->get_billing_last_name(),
				'email' => $order->get_billing_email(),
				'phone' => (string)$order->get_billing_phone(),
				'publisherId' => $this->options['fortvision_script_val'],
				'userId' => $this->userId,
				'isLoggedIn' => ($current_user->ID) ? 1 : 0,
			);
			//$return['status'] = $order->get_status();
			$return['userInfo'] = $userInfo;
			$return['cart'] = $order_cartparamas;
			
			return $return;
		}
		
		protected function getTaxonomy($productId, $taxonomyName) {
			$return = array();
			$terms = get_the_terms($productId, $taxonomyName);
			if ($terms && !is_wp_error($terms)) {
				
				foreach ($terms as $term) {
					$return[] = array(
						'id' => (string)$term->term_id,
						'description' => $term->description,
						'name' => $term->name,
						'url' => get_term_link($term),
					);
				}
			}
			
			return $return;
		}
		
		protected function getAttributes($productId) {
			$return = array();
			$product = wc_get_product($productId);
			if (!is_wp_error($product) && $product && $product->has_attributes()) {
				$attributes = $product->get_attributes();
				foreach ($attributes as $attribute) {
					if (!$attribute->get_visible()) {
						continue;
					}
					$taxObject = $attribute->get_taxonomy_object();
					$options = $attribute->get_options();
					$return[] = array(
						'id' => (string)$attribute->get_name(),
						'description' => (($options && is_array($options)) ? implode(', ', $options) : ''),
						'name' => $taxObject->attribute_name,
						'url' => '',
					);
				}
			}
			
			return $return;
		}
		
		protected function sendRequest($action, $data) {
			$curlResponse = $curlError = '-';
			if ($this->options['debug_mode_enabled_requests']) {
				$curlUrl = $this->routUrl_debug . $this->routes[$action];
			} else {
				$curlUrl = $this->routUrl . $this->routes[$action];
			}
			$curlUrl_log = $curlUrl;
			
			preg_match('|:(\d+)|', $curlUrl, $match);
			if ($match && count($match) == 2) {
				$curlUrl = str_replace($match[0], '', $curlUrl);
				$curlPort = $match[1];
			} else {
				$curlPort = false;
			}
			
			$ch = curl_init($curlUrl);
			if ($curlPort)
				curl_setopt($ch, CURLOPT_PORT, $curlPort);
			
			$method = 'PUT';
			if (in_array($action, array('addUserToSubscription')))
				$method = 'POST';
			
			curl_setopt($ch, CURLOPT_TIMEOUT, 3);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
			curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($data));
			
			curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);

			$verbose = fopen('php://temp', 'w+');
			curl_setopt($ch, CURLOPT_VERBOSE, true);
			curl_setopt($ch, CURLOPT_STDERR, $verbose);
			curl_setopt($ch, CURLINFO_HEADER_OUT, 1);
						
			$curlResponse = curl_exec($ch);
			$curlError = curl_error($ch);
			$curlCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
			
			if ($curlCode != 204) {
				rewind($verbose);
				$verboseLog = stream_get_contents($verbose);
			} else {
				$verboseLog = '-';
			}
			
			if ($this->options['debug_mode_enabled']) {
				$logData = array(
					'Time' => date('Y-m-d H:i:s'),
					'Action' => $action,
					'Method' => $method,
					'Rout' => $curlUrl_log,
					'Data' => $data,
					'Response' => $curlResponse,
					'Code' => $curlCode,
					'Error' => $curlError,
					'Verbose' => $verboseLog
					
				);
				$this->_log($logData);
			}
			
			curl_close($ch);
		}
		
		public function _log($data) {
			$logData = array();
			$uploadDir = wp_upload_dir(null, false);
			$logBaseDir = $uploadDir['basedir'] . '/fortvision-logs/';
			$logFile = $logBaseDir . 'requests.log';
			if (!file_exists($logBaseDir)) {
				mkdir($logBaseDir, 0755);
			}
			
			foreach ($data as $key => $value) {
				if (is_array($value)) {
					$value = var_export($value, true);
				}
				$logData[] = $key . ":\n" . $value;
			}
			@file_put_contents($logFile, implode("\n", $logData) . "\n\n", FILE_APPEND);
		}
		
		protected function logRotate() {
			$uploadDir = wp_upload_dir(null, false);
			$logBaseDir = $uploadDir['basedir'] . '/fortvision-logs/';
			$logFile = $logBaseDir . 'requests.log';
			if (file_exists($logFile) && @filesize($logFile) > 10 * 1024 * 1024) {
				@unlink($logFile);
			}
		}
	}