<?php
/**
 * index.php
 *
 * Copyright (c) 2016-2021 "fortvision" https://www.fortvision.com/
 *
 * This code is provided subject to the license granted.
 * Unauthorized use and distribution is prohibited.
 * See COPYRIGHT.txt and LICENSE.txt *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * This header and all notices must be kept intact.
 *
 * @author FORTVISION
 * @package fortvision-platform
 * @since fortvision-platform 1.0
 *
 * Plugin Name: FORTVISION Platform 
 * Plugin URI: https://www.fortvision.com/fortvision-wordpress-plugin
 * Version: 2.9.51
 * Description: The FORTVISION Lead-Gen Interactive Content Suite helps you amplify your message through awesome, engaging, interactive formats and unique placements that enhance, not distract, the user experience. No coding required!
 * Author: FORTVISION
 * Author URI: https://www.fortvision.com/
 * Text Domain: fortvision-platform
 */

namespace Fortvision\Fortvision_script_includer;
use \Fortvision\FortvisionEventTracker;

require_once __DIR__ . '/includes/event_tracker.php';

// to add fortvision menu page in admin
if (!defined('ABSPATH')) {
    die('Invalid request.');
}
define('FTVSN_LICENSE', true);
define('FTVSN_PLUGIN_DIR', __DIR__);
define('FTVSN_PLUGIN_FILE', __FILE__);
define('FTVSN_PLUGIN_VER', '2.9.51');

/**
     * @since 2.7.9
     * function cerTrace
     * for the output debug info in the content  
     * @param $data   : any
     * @param $isExit : boolean
*/
if(!function_exists("cerTrace")){   

    function cerTrace($data,$isExit=false){
       if($isExit=="comment") {
           echo "<!--".print_r($data,true)." -->";
           return;
       }
       echo "<pre>".print_r($data,true)."</pre>";
       if($isExit) exit;
    }    
}


if (!class_exists('Fortvision_script_includer')) {

    class Fortvision_script_includer {
    	
        private $options = array();
        private $isWoocommerce = false;

        function __construct() {
			$this->init();
			$this->register_hooks();
        }

        public function init() {
			$this->fortvision_getOptionData();
		}

		public function register_hooks() {
			add_action('init', array($this,'action_init'));
			add_action('after_setup_theme', array($this, 'fortvisionInitForm'), 1);
			add_action('admin_menu', array(&$this, 'fortvision_script_page_create'),1);
			register_activation_hook(__FILE__, array(&$this, 'fortvision_admin_notice_activation_hook'));
			
			add_action('admin_notices', array(&$this, 'fortvision_admin_noticer'));
			
			add_action('wp_enqueue_scripts', array(&$this, 'register_scripts'), 100);
			add_action('admin_enqueue_scripts', array(&$this, 'register_scripts'), 100);
			add_action('admin_enqueue_scripts', array(&$this, 'fortvision_loader_admin_style'));
		}
		
        function fortvision_getOptionData(){
			$this->options['fortvision_script_val'] = trim(get_option('fortvision_script_val', ''));
			$this->options['debug_mode_enabled'] = get_option('fortvision_debug_mode_enabled', '');
			$this->options['debug_mode_enabled_requests'] = get_option('fortvision_debug_mode_enabled_requests', '');
			$this->options['subscriptions'] = json_decode(get_option('_fortvision_form', '{}'), true);
        }

        public function action_init() {
			$this->isWoocommerce = is_plugin_active('woocommerce/woocommerce.php');
		}
		
        function fortvisionInitForm (){    
            if(!isset($this->options['subscriptions']))
            	$this->fortvision_getOptionData();  
            if(!empty($this->options['subscriptions']['which_pages']) 
                && 
                in_array($this->options['subscriptions']['which_pages'], array('on_checkout_page', 'on_signup_page', 'both')))
            {                
                
                if(in_array($this->options['subscriptions']['which_pages'] , array('on_checkout_page', 'both')))
                {
                    $pos = isset($this->options['subscriptions']['checkbox_location']) ? $this->options['subscriptions']['checkbox_location'] : 'after_order_record';                    
                    if($pos == 'after_order_record') {
                        add_action('woocommerce_after_order_notes',          array($this, 'fortvisionAddCheckBoxMarketing'));
                    } else {
                        add_action('woocommerce_review_order_before_submit', array($this, 'fortvisionAddCheckBoxMarketing'));
                    }
                } 
                
                if(in_array($this->options['subscriptions']['which_pages'], array('on_signup_page', 'both')))
                    add_action('woocommerce_register_form', array($this, 'fortvisionAddCheckBoxMarketing'));
            }  
        }

        function fortvisionAddCheckBoxMarketing() {
            $status = !empty($this->options['subscriptions']['checkbox_type']) ? true : false;
    
            if (is_user_logged_in() && metadata_exists( 'user', get_current_user_id(), '_fortvisionAcceptMarketing' ))
            {
                $status = get_user_meta(get_current_user_id(), '_fortvisionAcceptMarketing', true);
                if ((bool) $status)
                    return;
            }
    
            $label = !empty($this->options['subscriptions']['checkbox_text']) ? esc_html($this->options['subscriptions']['checkbox_text']) : __( 'I agree to receive promotional emails and text messages.', 'fortvision-platform');
    
			$checkbox = '<p class="form-row notes">';
			$checkbox .= '<label class="woocommerce-form__label woocommerce-form__label-for-checkbox checkbox">';
			$checkbox .= '<input type="checkbox" class="woocommerce-form__input woocommerce-form__input-checkbox input-checkbox" name="_fortvisionAcceptMarketing"  id="_fortvisionAcceptMarketing" value="1"' . ($status ? ' checked="checked"' : '') . ' />';
			$checkbox .= '<span class="woocommerce-terms-and-conditions-checkbox-text">' . $label . '</span>';
			$checkbox .= '</label>';
			$checkbox .= '</p>';
			
            echo $checkbox;
        }
        
        function fortvision_admin_notice_activation_hook() {
            set_transient('fortvision_admin_notice', true, 5);
        }

        function fortvision_admin_noticer() {
            if (get_transient('fortvision_admin_notice')) {
                ?>
                <div class="updated notice is-dismissible">
                    <p><strong>Thanks for installing! </strong>Now, head over to <a href="my.Fortvision.com">my.Fortvision.com</a> and create a Site Owner ID. You will receive your Site Id that you need to add below to start generating campaings</p>
                    <div class="add_fortvision_ids"><a target="_blank" href="https://my.fortvision.com/#/login">Add</a></div>
                    <p class="support_fortvision_com">If you have any questions, please contact <a href="mailto:support@fortvision.com">support@fortvision.com</a></p>
                </div>
                <?php
                delete_transient('fortvision_admin_notice');
            }
        }

        // to create submenu page
        function fortvision_script_page_create() {
            add_menu_page( 'Fortvision Plugin Settings', 'FORTVISION', 'manage_options', 'fortvision_settings', array(&$this, 'fortvision_scriptoption_page_display'),esc_url( plugins_url( 'images/fortvision-icon.png?v=' . FTVSN_PLUGIN_VER, __FILE__ ) ), 10 );
        }

        // setting in admin 
        function fortvision_scriptoption_page_display() {
            //esc_url(plugins_url('images/fortvision-logo.png', __FILE__))
            ?>
			<?php
			if (isset($_POST['fortvision_script_val']) && wp_verify_nonce($_POST['nonce'], 'wporg_options')) {
				if (isset($_POST['_fortvision'])) {
					update_option('_fortvision_form', json_encode($_POST['_fortvision']));
					$this->fortvision_getOptionData();
				}
				if (count(explode(' ', sanitize_text_field($_POST['fortvision_script_val']))) > 1) {
					echo '<div id="message" class="error notice notice-error is-dismissible"><p><strong>Please do not add any space in option value.</strong></p><button type="button" class="notice-dismiss"><span class="screen-reader-text">Dismiss this notice.</span></button></div>';
				} else {
					update_option('fortvision_script_val', sanitize_text_field($_POST['fortvision_script_val']));
					echo '<div id="message" class="updated notice notice-success is-dismissible"><p><strong>Options saved Successfully.</strong></p><button type="button" class="notice-dismiss"><span class="screen-reader-text">Dismiss this notice.</span></button></div>';
				}
			}
			$cidvalue = get_option('fortvision_script_val');
			?>
            <div class="fv_main_section">
                <div class="fv_header">
                    <img src="<?php echo esc_url(plugins_url('images/header.png', __FILE__))?>">
                </div>

                
                <div class="scriptingbox">
                        <p style="padding-left: 10px;"><?php echo __('Visit <a href="https://my.fortvision.com" target="_blank">my.fortvision.com</a>, navigate to Account tab to locate your Site ID', 'fortvision-platform'); ?></p>
                        <form method="post">
                            <table class="form-table">
                                <tr valign="top"><th scope="row" style="padding-left: 10px;"><?php echo esc_html(__('FORTVISION Status:', 'fortvision-platform')); ?></th>
                                    <td>
                                        <?php
                                        $fortvision_script_val = get_option('fortvision_script_val');
                                        $debug_mode_enabled = get_option('fortvision_debug_mode_enabled');
                                        $debug_mode_enabled_requests = get_option('fortvision_debug_mode_enabled_requests');
                                        $fortvision_script_val = trim($fortvision_script_val);
                                        $cidvalue = trim($cidvalue);
                                        if ($fortvision_script_val && $cidvalue) {
                                            echo "<strong class='activer'>Active</strong>";
                                        } else {
                                            echo "<strong class='deactiver'>Inactive</strong>";
                                        }
                                        ?>
                                    </td>
                                </tr> 
                                <tr valign="top"><th scope="row" style="padding-left: 10px;"><?php echo esc_html(__('Site ID:', 'fortvision-platform')); ?></th>
                                    <td><input required placeholder="Please Insert Value" type="text" name="fortvision_script_val" value="<?php echo $cidvalue; ?>" /><label><?php echo esc_html(__('Required')); ?></label></td>
                                <input type="hidden" name="nonce" value="<?php echo wp_create_nonce('wporg_options'); ?>" />
                                </tr>
                                <!--
								<tr>
									<td colspan="2"><?php echo (($debug_mode_enabled) ? '<strong style="color:green;">Logs ON</strong>' : '') ?></td>
								</tr>
								<tr>
									<td colspan="2"><?php echo (($debug_mode_enabled_requests) ? '<strong style="color:green;">Debug mode ON</strong>' : '') ?></td>
								</tr>
								-->
                            </table>

                            <?php if ($this->isWoocommerce) { ?>
							<table class="form-table">
								<tr>
									<td><h3 class="td-underline"><?php echo esc_html(__('Marketing Activity Setup:', 'fortvision-platform')); ?></h3></td>
								</tr>
								<tr>
									<td class="pt-0 pb-1"><p class="mt-0"><?php echo esc_html(__('FORTVISION can manage your subscription list by adding a checkbox to the pages you choose, or you can use your own checkbox.')); ?></p>
								</tr>
								<tr>
									<td class="pt-0 pb-0">
										<table class="form-table">
											<tr>
												<th class="pb-1 pt-0"><h4 class="mb-0 mt-0"><?php echo esc_html(__('On which page should the checkbox be displayed?', 'fortvision-platform')); ?></h4></th>
												<th class="pb-1 pt-0"></th>
											</tr>
											<tr>
												<td class="pt-0 va-t pb-0"">
                                                    <script>
                                                        function switchOnOffFortvision (status){                                                           
                                                           jQuery('.fortvision_form_element').prop( "disabled", status );
                                                           if(status) {
                                                               jQuery('.fortvision_form_element').prop( "checked",!status);
                                                               jQuery('.fortvisiondef').prop( "checked",status);
                                                           }                                                           
                                                        }
                                                    </script>    
													<input type="radio"
														   name="_fortvision[which_pages]"                                                           
														   value="0" <?php
															echo ( empty($this->options['subscriptions']['which_pages']) 
                                                                   || $this->options['subscriptions']['which_pages']=='0'
																 ) ? ' checked="checked" ' : '';
														
														?> 
                                                            onclick="switchOnOffFortvision(true);" /><label class="d-i mt--15"><?php echo esc_html(__('Disable')); ?></label><br/>
													<input type="radio"
														   name="_fortvision[which_pages]"                                                           
														   value="on_checkout_page"
														<?php
															echo (!empty($this->options['subscriptions']['which_pages'])
																&&
																$this->options['subscriptions']['which_pages'] == 'on_checkout_page' ) ? ' checked="checked" ' : '';
														
														?>
                                                        onclick="switchOnOffFortvision(false);" /><label class="d-i mt--15"><?php echo esc_html(__('On checkout page (when user fille order information)')); ?></label><br/>
													<input type="radio"
														   name="_fortvision[which_pages]"                                                           
														   value="on_signup_page"
														<?php
															echo (!empty($this->options['subscriptions']['which_pages'] )
																&&
																$this->options['subscriptions']['which_pages'] == 'on_signup_page' ) ? ' checked="checked" ' : '';
														?> 
                                                        onclick="switchOnOffFortvision(false);" /><label class="d-i mt--15"><?php echo esc_html(__('On signup page (when user signup)')); ?></label><br/>
													<input type="radio"
														   name="_fortvision[which_pages]"                                                           
														   value="both"
														<?php
															echo (!empty($this->options['subscriptions']['which_pages'])
																&&
																$this->options['subscriptions']['which_pages'] == 'both' ) ? ' checked="checked" ' : '';
														?> 
                                                        onclick="switchOnOffFortvision(false);"  /><label class="d-i mt--15"><?php echo esc_html(__('Both (signup and order forms)')); ?></label><br/>
												</td>
												
												<td class="pt-0 va-t pb-0"">
													
													<input type="checkbox"
														   name="_fortvision[checkbox_type]"
                                                           class="fortvision_form_element fortvisiondef"
														   value="checked_on_default"
														<?php                                                        
															echo (
                                                                !isset($this->options['subscriptions']['checkbox_type']) 
                                                                ||
																!empty($this->options['subscriptions']['checkbox_type'])
																&&
																$this->options['subscriptions']['checkbox_type'] == 'checked_on_default' ) ? ' checked="checked" ' : '';
														
														?> /><label class="d-i mt--15"><?php echo esc_html(__('checked by default')); ?></label><br/>
												</td>
											</tr>
										</table>
									</td>
								</tr>
								<tr>
									<td class="pb-0 pt-0"">
										<h4 class="mb-0"><?php echo esc_html(__('Checkbox text:', 'fortvision-platform')); ?></h4>
									</td>
								</tr>
								<tr>
									<td>
                                        
                                        <textarea 
                                                class="width-100" 
                                                name="_fortvision[checkbox_text]"
                                                rows="4"
                                                placeholder="<?php echo esc_html( __('e.g: i allow to get promotions, updates and benefits by email and sms') ) ; ?>"
                                        ><?php
												echo (!empty($this->options['subscriptions']['checkbox_text']) ) ? $this->options['subscriptions']['checkbox_text'] : 'I agree I want to receive news, offers, tips and other promotional materials, including my email and phone I am submitting. I understand that I can withdraw my consent and unsubscribe at any time.';
											?>
                                            
                                        </textarea>
                                        
									</td>
								</tr>
								<?php /* tr>
									<td class="pb-0"><h2 class="td-underline"><?php echo esc_html(__('When to report a purchase?', 'fortvision-platform')); ?></h2></td>
								</tr>
								<tr>
									<td>
										<input type="hidden"
											   name="_fortvision[when_to_report_a_purchase]"
											   value="0" />
										<input type="radio"
											   name="_fortvision[when_to_report_a_purchase]"
											   value="when_arriving_to_thank_you_page"
                                               <?php
													 echo (
																!empty($this->options['subscriptions']['when_to_report_a_purchase'])
																&&
																$this->options['subscriptions']['when_to_report_a_purchase'] == 'when_arriving_to_thank_you_page' ) ? ' checked="checked" ' : '';
														
														?>
                                        />
										<label class="d-i mt--15"><?php echo esc_html(__('When arriving to “Thank you” page', 'fortvision-platform')); ?></label>
										<br>										
										<input type="radio"
											   name="_fortvision[when_to_report_a_purchase]"
											   value="when_order_has_been_placed"
                                               <?php
															echo (
																!empty($this->options['subscriptions']['when_to_report_a_purchase'])
																&&
																$this->options['subscriptions']['when_to_report_a_purchase'] == 'when_order_has_been_placed' ) ? ' checked="checked" ' : '';
														
														?>
                                        />
										<label class="d-i mt--15"><?php echo esc_html(__('When order has been placed', 'fortvision-platform')); ?></label>										
									</td>
								</tr */ ?>
								<?php do_action('fortvision_after_settings_form'); ?>
							</table>
                           <?php } else {
                            	echo '<table class="form-table">';
								do_action('fortvision_after_settings_form');
                            	echo '</table>';
							}
							?>
							
                            <p class="submit">
                                <input type="submit" name="saveFortvisionSettings" class="button-primary" value="<?php _e('Save Changes'); ?>" />
                            </p>
                        </form>
                        <!--<p><?php _e('If you have any questions or suggestions about this plugin, reach out to us at '); ?><a href="mailto:support@fortvision.com"><?php echo _e('support@fortvision.com'); ?></a></p>-->
                    </div>
                <div class="fv_footer">
                    <img src="<?php echo esc_url(plugins_url('images/footer.png', __FILE__))?>">
                </div>
            </div>
            <?php
        }

        function fortvision_loader_admin_style() {
            wp_register_style('cadmin_css', esc_url(plugins_url('assets/custom.min.css', __FILE__)), false, FTVSN_PLUGIN_VER);
            wp_enqueue_style('cadmin_css', esc_url(plugins_url('assets/custom.min.css', __FILE__)), false, FTVSN_PLUGIN_VER);
        }
        
		public function register_scripts() {
			$output = array();
			
			wp_register_script('fortvision_events', esc_url(plugins_url('assets/events.js', FTVSN_PLUGIN_FILE)), array(), time(), true);
			$translation_array = array(
				'fortvision' => $this->options['fortvision_script_val'],
				'fortvision_url' => esc_url('https://fortcdn.com/staticfiles/fb-web/js/fortvision-fb-web-cms-plugin.js'),
				'version' => FTVSN_PLUGIN_VER,
			);
			if ($this->options['debug_mode_enabled_requests']) {
				$translation_array['fortvision_without_url'] = esc_url('https://s3.eu-west-1.amazonaws.com/resources-dev.fortvision.com/staticfiles/fb-web/js/fortvision-fb-web.js');
			} else {
				$translation_array['fortvision_without_url'] = esc_url('https://fortcdn.com/staticfiles/fb-web/js/fortvision-fb-web.js');
			}
			
			wp_localize_script('fortvision_events', 'fortvision_object_name', $translation_array);
			
			wp_enqueue_script('fortvision_events');
		}
    }

    $s = new \Fortvision\Fortvision_script_includer\Fortvision_script_includer();
    $FortvisionEventTrackerObject = \Fortvision\FortvisionEventTracker\FortvisionEventTracker::instance();
}